use std::sync::mpsc;
use std::thread;

#[derive(Debug)]
enum Message {
   Integer(i32),
   Float(f64),
   String(String),
}

fn main() {
   let (tx, rx) = mpsc::channel();

   // Spawn thread 1 (handles integers)
   let tx1 = tx.clone();
   thread::spawn(move || {
      while let Ok(message) = rx.recv() {
         match message {
            Message::Integer(num) => {
               println!("Thread 1: Handling integer: {}", num);
            }
            other => {
               println!("Thread 1: Cannot handle {:?}, passing it on.", other);
               tx1.send(other).unwrap();
            }
         }
      }
   });

   // Spawn thread 2 (handles floats)
   let (tx2, rx2) = mpsc::channel();
   thread::spawn(move || {
      while let Ok(message) = rx2.recv() {
         match message {
            Message::Float(num) => {
               println!("Thread 2: Handling float: {}", num);
            }
            other => {
               println!("Thread 2: Cannot handle {:?}, passing it on.", other);
               tx2.send(other).unwrap();
            }
         }
      }
   });

   // Spawn thread 3 (handles strings starting with "EVENT: ")
   thread::spawn(move || {
      while let Ok(message) = tx2.recv() {
         match message {
            Message::String(s) if s.starts_with("EVENT: ") => {
               println!("Thread 3: Handling event string: {}", s);
            }
            Message::String(s) => {
               println!("Thread 3: Cannot handle string '{}', error!", s);
            }
            other => {
               println!("Thread 3: Unhandled message {:?}, error!", other);
            }
         }
      }
   });

   // Main thread: Get user input and send messages
   loop {
      let mut input = String::new();
      println!("Enter a message (integer, float, or string starting with 'EVENT: '):");
      if std::io::stdin().read_line(&mut input).is_err() {
         println!("Failed to read input.");
         continue;
      }
      let input = input.trim();
      if input.is_empty() {
         break; // Exit loop on empty input
      }

      let message = if let Ok(num) = input.parse::<i32>() {
         Message::Integer(num)
      } else if let Ok(num) = input.parse::<f64>() {
         Message::Float(num)
      } else {
         Message::String(input.to_string())
      };

      if let Err(err) = tx.send(message) {
         println!("Failed to send message: {}", err);
         break;
      }
   }

   println!("Exiting program.");
}

