// [dependencies]
// tokio = { version = "1", features = ["full"] }
// hyper = { version = "0.14", features = ["full"] }
// futures = "0.3"

use hyper::{service::{make_service_fn, service_fn}, Body, Request, Response, Server};
use tokio::time::{sleep, Duration};
use std::convert::Infallible;

// Simulate a slow I/O task (e.g., reading from a database)
async fn slow_task(query: String) -> String {
    println!("Started processing query: {}", query);
    sleep(Duration::from_secs(2)).await;  // Simulating a 2-second delay
    println!("Finished processing query: {}", query);
    format!("Processed: {}", query)
}

// The function that will handle each HTTP request
async fn handle_request(req: Request<Body>) -> Result<Response<Body>, Infallible> {
    // Get the query parameters or use a default if none provided
    let query = req.uri().query().unwrap_or("No query parameter");

    // Simulate async work
    let result = slow_task(query.to_string()).await;

    // Return the result as the HTTP response
    Ok(Response::new(Body::from(result)))
}

#[tokio::main]
async fn main() {
    // Define the address to bind the server
    let addr = ([127, 0, 0, 1], 8080).into();

    // Create the service that will handle HTTP requests
    let make_svc = make_service_fn(|_conn| async {
        Ok::<_, Infallible>(service_fn(handle_request))
    });

    // Create the HTTP server
    let server = Server::bind(&addr).serve(make_svc);

    // Output the server address and start serving
    println!("Server running at http://127.0.0.1:8080");
    
    // Run the server until it's closed or terminated
    if let Err(e) = server.await {
        eprintln!("Server error: {}", e);
    }
}


